function [T_EA2LR,actT,actE,actA,actL,actR,act,T_LR2EA] = actdef(varargin)
%[T_EA2LR,actT,actE,actA,actL,actR,act,T_LR2EA] = actdef(varargin)
%ACTDEF defines models describing the actuator dynamics. The throttle
%actuator is defined as a first-order system with a BW of 1 Hz. The control
%surface actuators may be defined as either first- or second-order systems.
%The order is specified using the first input argument.
%The control surface actuators are further defined based on the second
%input argument. In particular, if the second input argument is:
% 'TD'      A pure time delay of 0.05 s is set at the input.
% 'pade'    The time delay is approximated using a fifth-order pade model.
% 'LOES'    The actuator and time delay are approximated using a low-order
%           equivalent system.
% The third input argument control screen output. TRUE produces screen
% output and FALSE suppresses screen output.

%% Setup
switch nargin % switch based on number of input arguments
    case 0
        ord = 2; % default to second-order actuator
        mde = 'TD'; % default to the mode 'TD'
        vb = false; % suppress screen output
    case 1
        ord = varargin{1}; % set actuator order
        mde = 'TD'; % default to the mode 'TD'
        vb = false; % suppress screen output
    case 2
        ord = varargin{1}; % set actuator order
        mde = varargin{2}; % set time delay mode
        vb = false; % suppress screen output
    case 3
        ord = varargin{1}; % set actuator order
        mde = varargin{2}; % set time delay mode
        vb = varargin{3}; % suppress screen output
    otherwise
        error('Incorrect number of inputs.');
end
TD = 0.05; % default to time delay of 0.05 s
if vb; fprintf('\n - The actuator order is %d and the time delay is %2.2f seconds.\n',ord,TD); end;

%% Transform (L/R) <-- (E/A)
T_EA2LR = ss([1,-1;1,1]);
set(T_EA2LR,'InputName',{'elevator';'aileron'},'OutputName','CtrlSurf');

%% Transform (E/A) <-- (L/R)
T_LR2EA = ss(inv(T_EA2LR.d));
set(T_LR2EA,'InputName',{'l_elevon_cmd','r_elevon_cmd'},'OutputName',{'elevator_cmd','aileron_cmd'});

%% Throttle actuator model
BW_T = 2*pi*1; % BW of throttle [rad/s]
actT_TD = ss(tf(BW_T,[1,BW_T])); actT_TD.InputDelay = TD;
actTP2 = pade(actT_TD,2); % second-order pade approximation of time-delay

%% Control surface actuators
% Set actuator model based on order requested
switch ord
    case 1 % first-order actuator
        act = ss(tf(50.27,[1,50.27]));
    case 2 % second-order actuator
        act = ss(tf(3940,[1,97.05,3940]));
    otherwise % anything else requested
        error('I can only provide first- or second-order actuator models.');
end

% Create exact, pade, and LOES models
act.InputDelay = TD; % exact modeling of time-delay
actP5 = pade(act,5); % fifth-order pade approximation of time-delay
[Gbal,~] = balreal(actP5); % balanced realization of actP5
n = order(Gbal); % order of fifth-order pade approx for TD + act
nr = 2; % number of states to residualize
indResid = (n-nr+1):n; % indices of states to be residualized
LOES = modred(Gbal,indResid); % low-order equivalent system of actP5

% Set time delay model based on method requested
switch lower(mde)
    case 'td'
        if vb; fprintf(' - The time-delay is modeled exactly.\n'); end;
        actT = actT_TD; actE = act; actA = act; actL = act; actR = act;
    case 'pade'
        if vb; fprintf(' - The time-delay is modeled using a fifth-orde pade model.\n'); end;
        actT = actTP2; actE = actP5; actA = actP5; actL = actP5; actR = actP5;
    case 'loes'
        if vb; fprintf(' - The time-delay is modeled using a low-order equivalent system.\n'); end;
        actT = actTP2; actE = LOES; actA = LOES; actL = LOES; actR = LOES;
    otherwise
        error(['Incorrect time-delay model "',mde,'".']);
end

% Set properties
set(actT,'InputName','throttle_cmd','OutputName','throttle',...
    'Name','Throttle actuator'); actT.StateName(:) = {'xT'};

set(actE,'InputName','elevator_cmd','OutputName','elevator',...
    'Name','Elevator actuator'); actE.StateName(:) = {'xE'};

set(actA,'InputName','aileron_cmd','OutputName','aileron',...
    'Name','Aileron actuator'); actA.StateName(:) = {'xA'};

set(actL,'InputName','l_elevon_cmd','OutputName','CtrlSurf(1)',...
    'Name','Left elevon actuator'); actL.StateName(:) = {'xL'};

set(actR,'InputName','r_elevon_cmd','OutputName','CtrlSurf(2)',...
    'Name','Right elevon actuator'); actR.StateName(:) = {'xR'};

set(act,'InputName','command','OutputName','position',...
    'Name','KST DS135MG actuator'); act.StateName(:) = {'act'};
end
